(**************************************************************************)
(*                                                                        *)
(*  Ocamlviz --- real-time profiling tools for Objective Caml             *)
(*  Copyright (C) by INRIA - CNRS - Universite Paris Sud                  *)
(*  Authors: Julien Robert                                                *)
(*           Guillaume Von Tokarski                                       *)
(*           Sylvain Conchon                                              *)
(*           Jean-Christophe Filliatre                                    *)
(*           Fabrice Le Fessant                                           *)
(*  GNU Library General Public License version 2                          *)
(*  See file LICENSE for details                                          *)
(*                                                                        *)
(**************************************************************************)

open Mlpost
open Num
open Command
open Helpers
open Box

let init1 = 
  let node name dx dy fill = 
    Box.round_rect ~name ~dy ~stroke:(Some Color.black) ~fill 
      (Box.tex ("\\phantom{pI}" ^ name ^ "\\phantom{pI}"))
  in
  let program = node "programme" (bp 5.) (bp 5.) Color.lightgreen in
  let ocamlviz = node "\\tt Ocamlviz" (bp 5.) (bp 5.) Color.yellow in
  let net = node "\\tt Net" (bp 5.) (bp 5.) Color.lightblue in
  let database = node "\\tt Db" (bp 5.) (bp 5.) Color.lightgray in
  let display = node "affichage" (bp 5.) (bp 5.) Color.orange in
  let l = [program;ocamlviz;net;database;display] in
  match same_height l with
    | [a;b;c;d;e] -> a,b,c,d,e
    | _ -> assert false
  
let archi1 = 
  let program,ocamlviz,net,database,display = init1 in
  let boxstructure = Array.make_matrix 4 2 (Box.empty ()) in
  let boxserver = Array.make_matrix 1 2 (Box.empty ()) in
  let boxclient = Array.make_matrix 1 3 (Box.empty ()) in
  boxserver.(0).(0) <-  program ;
  boxserver.(0).(1) <-  ocamlviz ;
  boxclient.(0).(0) <-  net ;
  boxclient.(0).(1) <-  database ;
  boxclient.(0).(2) <-  display ;
  let server = Box.tabular ~name:"server" boxserver in
  let client = Box.tabular ~name:"client" boxclient in
  let clientn = Box.tabular ~name:"clientn" boxclient in
  boxstructure.(0).(0) <- Box.tex "serveur";
  boxstructure.(0).(1) <- Box.tex "clients";
  boxstructure.(1).(0) <- server ;
  boxstructure.(1).(1) <- client ;
  boxstructure.(2).(1) <- Box.rect ~name:"etc" ~stroke:None ~dy:(bp 10.) (Box.tex ".  .  .") ;
  boxstructure.(3).(1) <- clientn ;
  let structure = Box.tabular ~vpadding:(bp 10.) ~hpadding:(bp 100.) boxstructure in
  let p1 = Box.east (Box.get "server" structure) in
  let p2 = Box.west (Box.get "client" structure) in
  let p3 = Box.west (Box.get "etc" structure) in
  let p4 = Box.west (Box.get "clientn" structure) in
  let xp1 = Point.xpart p1 in
  let yp1 = Point.ypart p1 in
  let xpic = Num.addn (bp 15.) xp1 in
  let ypic = Num.addn yp1 (bp 10.) in
  let pic = Picture.shift (Point.pt (xpic,ypic)) (Picture.tex "protocole binaire") in
  let a1 = Arrow.simple (Path.pathp [p1;p2]) in
  let a2 = Arrow.simple (Path.pathp [p1;p3]) in
  let a3 = Arrow.simple (Path.pathp [p1;p4]) in
  (Box.draw structure)++a1++a2++a3++(Picture.make pic)




let tabular l =
  "\\begin{tabular}{c}" ^ 
  String.concat "\\\\" l ^
  "\n\\end{tabular}"

let init2 = 
  let node name dis dx dy fill = Box.round_rect ~name ~dx ~dy ~stroke:(Some Color.black) ~fill (Box.tex dis) in
  let program = node "prog" (tabular ["program"; "instrumented"]) (bp 5.) (bp 10.) Color.lightgreen in
  let ocamlviz = node "ocamlviz" (tabular ["library";"Ocamlviz"]) (bp 5.) (bp 10.) Color.yellow in
  let display = node "display" (tabular ["GUI";"Ocamlviz"]) (bp 5.) (bp 10.) Color.lightblue in
  (program,ocamlviz,display)
  

let archi2 = 
  let program,ocamlviz,display = init2 in
  let leftpart = Array.make_matrix 1 2 (Box.empty ()) in
  let boxstructure = Array.make_matrix 1 3 (Box.empty ()) in
  leftpart.(0).(0) <- program ;
  leftpart.(0).(1) <- ocamlviz ;
  let left = Box.tabular ~name:"left" leftpart in
  boxstructure.(0).(0) <- left ;
  boxstructure.(0).(2) <- display ;
  let structure = Box.tabular ~hpadding:(bp 50.) boxstructure in
  let p1 = Box.east (Box.get "left" left) in
  let p2 = Box.west (Box.get "display" structure) in
  let xp1 = Point.xpart p1 in
  let xp1' = Num.addn xp1 (bp 5.) in
  let yp1 = Point.ypart p1 in
  let p1' = Point.pt (xp1',yp1) in
  let xp2 = Point.xpart p2 in
  let xp2' = Num.subn xp2 (bp 5.) in
  let yp2 = Point.ypart p2 in
  let p2' = Point.pt (xp2',yp2) in
  let xpic = Num.addn (bp 30.) xp1 in
  let ypic = Num.addn yp1 (bp 10.) in
  let pic = Picture.shift (Point.pt (xpic,ypic)) (Picture.tex "network") in
  let a1 = Arrow.draw (Path.pathp [p1';p2']) in
  (Box.draw structure)++a1++(Picture.make pic)


let init3 = 
  let node name dx dy fill = Box.round_rect ~name ~dx ~dy ~stroke:(Some Color.black) ~fill(*(Color.rgb8 102 170 51)*) (Box.tex name) in
  let program = node (tabular ["program";"instrumented"]) (bp 5.) (bp 10.) Color.lightgreen in
  let p4 = node "camlp4" (bp 10.) (bp 5.) Color.orange in
  let alarm = node "alarms" (bp 12.5) (bp 6.) Color.lightblue in
  let thread = node "threads" (bp 13.) (bp 7.) Color.lightblue in
  let server = node "server" (bp 10.) (bp 20.) Color.yellow in
  let client = node "client" (bp 10.) (bp 20.) Color.yellow in
  let database = node "\\tt Db" (bp 10.) (bp 20.) Color.lightgray in
  let gtk = node "GTK" (bp 12.5) (bp 7.) Color.lightblue in
  let ascii = node "ASCII" (bp 10.) (bp 7.) Color.lightblue in
  (program,p4,alarm,thread,server,client,database,gtk,ascii)


let archi3 =
  let program,p4,alarm,thread,server,client,database,gtk,ascii = init3 in
  let leftpart = Array.make_matrix 1 3 (Box.empty ()) in
  let rightpart = Array.make_matrix 1 3 (Box.empty ()) in
  let a_t = Array.make_matrix 2 1 (Box.empty ()) in
  let g_a = Array.make_matrix 2 1 (Box.empty ()) in
  let boxstructure = Array.make_matrix 2 2 (Box.empty ()) in
  let underpart = Array.make_matrix 1 3 (Box.empty ()) in
  a_t.(0).(0) <- alarm ;   
  a_t.(1).(0) <- thread ;
  let libraries = Box.tabular ~name:"libraries" a_t in 
  g_a.(0).(0) <- gtk ;   
  g_a.(1).(0) <- ascii ;
  let display = Box.tabular ~name:"display" g_a in
  leftpart.(0).(0) <- program ;
  leftpart.(0).(1) <- libraries ;
  leftpart.(0).(2) <- server ;
  rightpart.(0).(0) <- client ;
  rightpart.(0).(1) <- database ;
  rightpart.(0).(2) <- display ;
  let left = Box.tabular ~name:"left" leftpart in
  let right = Box.tabular ~name:"right" rightpart in
  underpart.(0).(0) <- p4;
  let under = Box.tabular ~hpadding:(bp 53.) ~name:"under" underpart in
  boxstructure.(1).(0) <- under ;
  boxstructure.(0).(0) <- left ;
  boxstructure.(0).(1) <- right ;
  let structure = Box.tabular ~hpadding:(bp 100.) boxstructure in
  let p1 = Box.east (Box.get "left" left) in
  let xp1 = Point.xpart p1 in
  let xp1' = Num.addn xp1 (bp 5.) in
  let yp1 = Point.ypart p1 in
  let p1' = Point.pt (xp1',yp1) in
  let xp2 = Num.addn xp1 (bp 95.) in
  let p2 = Point.pt (xp2,yp1) in
  let xpic = Num.addn (bp 15.) xp1 in
  let ypic = Num.addn yp1 (bp 10.) in
  let pic = Picture.shift (Point.pt (xpic,ypic)) (Picture.tex "binary protocol") in
  let a1 = Arrow.draw (Path.pathp [p1';p2]) in
  (Box.draw structure)++a1++(Picture.make pic)



let _ =
  List.iter (fun (name,fig) -> Metapost.emit name fig)
    [
      "archi",archi1;
      "simple_archi",archi2;
      "complex_archi",archi3;
    ]

(*
Local Variables: 
compile-command: "TEXINPUTS=`pwd`: mlpost -native -pdf -latex main-en-final.tex figures.ml"
End: 
*)
